function [Z0, gamma] = BT1_model(secGauge, freq)


%==========================================================================
%                           BT#1 CABLE MODEL
%                                                                         
% Description:                                                            
%   This function provides the secondary parameters and gammaL of each
%   line section that characterize the line as a whole based on British 
%   Telecom (BT) 1 model.
%==========================================================================
% Input parameter(s):
%   secGauge       Row-vector containing the gauge sequence for the line 
%                  under test, from CP to CO.                          [mm]
%   freq           Column-vector containing the frequency samples related  
%                  to the measured or simulated data.               [Hertz]
%==========================================================================
% Output parameter(s):
%   Z0             Characteristic impedance per line section. Matrix with 
%                  dimension (frequency samples x no. of sections).  [Ohms]
%   gamma          Propagation constant per line section. Matrix with
%                  dimension (frequency samples x no. of sections).  [1/km]
%==========================================================================

%==========================================================================
% Author(s):
%   Roberto Menezes (RoMe / menegues@ufpa.br / UFPA)
%==========================================================================
% Change history (yyyy-mm-dd):
%   2006-04-27    (RoMe)        Created.
%==========================================================================
% Version: ??
%==========================================================================
% Known bugs (-) or limitations (*):                                      
%   None detected yet.
%==========================================================================
% Copyright (c) 2006 UFPA, Federal University of Para, Brazil.
%==========================================================================


%% ==== DEFINITION AND PRE-ALLOCATION OF VARIABLES ========================


%=== Angular frequency: 

w = 2*pi*freq;                 


%=== No. of samples and sections: 

nSec = size(secGauge, 2);
nSamples = size(freq, 1);


%=== Pre-allocation for the primary and secundary parameters:

R = zeros(nSamples, nSec);
L = zeros(nSamples, nSec);
C = zeros(nSamples, nSec);
G = zeros(nSamples, nSec);

Zs = zeros(nSamples, nSec);
Yp = zeros(nSamples, nSec);
    
Z0 = zeros(nSamples, nSec);
gamma = zeros(nSamples, nSec);


%% ==== BT1 PARAMETERS PER GAUGE ==========================================
  
for i=1:nSec    % For each line section... 
        
    if secGauge(i) == 0.32 || secGauge(i) == 28
      r_0c = 0.4090e3;    r_0s = Inf;          a_c = 0.3822;     
      a_s = 0;            l_0 = 0.6075e-3;     l_inf = 0.5e-3;       
      f_m = 0.6090e6;     b = 5.269;           c_inf = 40e-9;        
      c_0 = 0;            c_e = 0;             g_0 = 0;    
      g_e = 0;

    elseif secGauge(i) == 0.40 || secGauge(i) == 26
      r_0c = 0.280e3;     r_0s = Inf;          a_c = 0.0969;     
      a_s = 0;            l_0 = 0.5873e-3;     l_inf = 0.4260e-3;    
      f_m = 0.7459e6;     b = 1.385;           c_inf = 50e-9;        
      c_0 = 0;            c_e = 0;             g_0 = 0;    
      g_e = 0;

    elseif secGauge(i) == 0.50 || secGauge(i) == 24
      r_0c = 0.1792e3;    r_0s = Inf;          a_c = 0.0561;     
      a_s = 0;            l_0 = 0.6746e-3;     l_inf = 0.5327e-3;    
      f_m = 0.6647e6;     b = 1.195;           c_inf = 50e-9;        
      c_0 = 0;            c_e = 0;             g_0 = 0;         
      g_e = 0;

    elseif secGauge(i) == 0.63 || secGauge(i) == 0.65 || secGauge(i) == 22
      r_0c = 0.113e3;     r_0s = Inf;          a_c = 0.0257;     
      a_s = 0;            l_0 = 0.6994e-3;     l_inf = 0.4772e-3;    
      f_m = 0.2658e6;     b = 1.0956;          c_inf = 45e-9;        
      c_0 = 0;            c_e = 0;             g_0 = 0;         
      g_e = 0;

    elseif secGauge(i) == 0.90 || secGauge(i) == 19
      r_0c = 0.0551e3;    r_0s = Inf;          a_c = 0.0094;     
      a_s = 0;            l_0 = 0.7509e-3;     l_inf = 0.5205e-3;    
      f_m = 0.1238e6;     b = 0.9604;          c_inf = 40e-9;        
      c_0 = 0;            c_e = 0;             g_0 = 0;         
      g_e = 0;
  
    else
      error(['Wrong cable_type!: ','section ', num2str(i)]);
      
    end

    
  %=== Primary parameters: 

    R(:,i) = 1 ./ ( ( 1 ./ sqrt(sqrt(r_0c^4 + a_c*freq.^2)) ) +...
            ( 1 ./ sqrt(sqrt(r_0s^4 + a_s*freq.^2)) ) );
    L(:,i) = ( l_0 + l_inf*((freq/f_m).^b) ) ./ ( 1 + (freq/f_m).^b );
    C(:,i) = c_inf + c_0 * freq.^(-c_e);
    G(:,i) = g_0 * freq.^g_e;


  %=== Characteristic impedance and gamma*leng: 
  
    Zs(:,i) = R(:,i) + 1i*w.*L(:,i);
    Yp(:,i) = G(:,i) + 1i*w.*C(:,i);
    
    Z0(:,i) = sqrt(Zs(:,i)./Yp(:,i));
    gamma(:,i) = sqrt( Zs(:,i).*Yp(:,i) );  % 1/km.    
  

end 



                          % END OF CODE %                          
