function [Z0, gamma] = VUB0_model(secGauge, freq, varargin)


%==========================================================================
%                         VUB0 CABLE MODEL
%                                                                         
% Description:                                                            
%   This function provides the the secondary parameters and gammaL of each
%   line section from a given VUB parameter vector (theta).
%==========================================================================
% Input parameter(s):
%   secGauge       Vector containing the gauge sequence for the line under
%                  test, from CP to CO.                                [mm]
%   freq           Frequency range.                               [kHz-MHz]
%   cablePar       Structure containing updated values for the geometric &
%                  electromagnetic parameters of the cable under 
%                  test. Specifically the fields are:
%                   a) radius - radius of the conductor.                [m]
%                   c) D2a - ratio D/2*radius.              [dimensionless]
%                   b) distCenters - distance between the 
%                      conductor centers.                               [m]
%                   d) relPermittivity - relative electric    
%                      permittivity of the dielectric.      [dimensionless]
%                   e) permittivity0 - free space electric 
%                      permittivity of the dieletric.                 [F/m]
%                   f) relPermeability - relative magnetic 
%                      permeability of the dieletric.       [dimensionless]
%                   g) permeability0 - relative magnetic 
%                      permeability of the dieletric.                 [H/m]
%                   h) conducttivity - conductor's conductivity.      [S/m]
%==========================================================================
% Output parameter(s):
%   Z0             Characteristic impedance per line section. Matrix with 
%                  dimension (sample frequency x no. of sections).   [Ohms]
%   gamma          Propagation constant per line section. Matrix with     
%                  dimension (sample frequency x no. of sections).   [1/km]
%  theta           Parameter matrix. Each i-th row of the matrix contains
%                  the VUB parameters that describes the i-th line section.
%==========================================================================

%==========================================================================
% Author(s):
%   Licinius Alcantara (LiAl / licinius@ufpa.br / UFPA)
%   Roberto Menezes (RoMe / menegues@ufpa.br / UFPA)
%==========================================================================
% Change history (yyyy-mm-dd):
%   2005-03-14    (LiAn/RoMe)   Created.
%   2005-04-06    (LiAn)        Extraction of ABCD matrix of the line from 
%                               VUB0 cable model.
%   2005-04-17    (RoMe)        Review of code structure to handle the new 
%                               P format (vector).
%   2005-06-23    (RoMe)        New review of code structure to handle the 
%                               new P format (matrix again).
%   2005-11-05    (RoMe)        Solved the error related to bad indexing 
%                               of P0 matrix.
%   2009-11-05    (RoMe)        Complete reformulation of code structure.
%                               Exclusion the variable "indexP0" and 
%                               re-formatting of the code according to 
%                               predefined coding convenctions.
%   2009-11-05    (RoMe)        Re-formatting of the code according to 
%                               predefined coding convenctions and change 
%                               the name of some variables (P0 -> theta;
%                               indexP0 -> secInfo).
%==========================================================================
% Version: ??
%==========================================================================
% Known bugs (-) or limitations (*):                                      
%   None detected yet.
%==========================================================================
% Copyright (c) 2005 UFPA, Federal University of Para, Brazil.
%==========================================================================


%% ==== DEFINITION AND PRE-ALLOCATION OF VARIABLES ========================

w = 2*pi*freq;  % Angular frequency.   
s = 1i*w;  % Complex frequency.    
nSamples = size(freq, 1);  % Number of frequency samples.
nSec = size(secGauge, 2);  % Number of line sections.


%=== Pre-allocation of the secundary parameters and "theta" vector:
    
Z0 = zeros(nSamples, nSec);
gamma = zeros(nSamples, nSec);
theta = zeros(nSec, 5);


%% ==== FILLING OF THE VECTOR PARAMETER (THETA) ===========================

for i = 1:nSec
              
     %--- Geometric & electromagnetic cable parameters
        
       if nargin < 3  % If user does not provide them, take the nominal...
                      %...values from cable database.
           [cablePar] = cable_parameters( secGauge(i) );         
       end
     
     %--- translate physical parameter in metters to km  
       a = cablePar.radius/1000;     
       D = cablePar.distCenters/1000;
       CONDUC = cablePar.conductivity*1000;   
       MI = (cablePar.permeability0*1000)*cablePar.relPermeability;
       EP = (cablePar.permittivity0*1000)*cablePar.relPermittivity;
        
     %--- VUB parameters (Theta vector)
       delta = EP/acosh( D/(2*a) );
       theta(i,1) = ( 1/a )*sqrt(MI/CONDUC)*delta;
       theta(i,2) = (a/D)^2;
       theta(i,3) = a*sqrt(MI*CONDUC);
       theta(i,4) = ( theta(i,1)*theta(i,3) )*log( 1/sqrt(theta(i,2)) );
       theta(i,5) = pi*delta;
           
       
%% ==== COMPUTATION OF THE SECONDARY PARAMETERS ===========================       
     
        J0 = besselj( 0, theta(i,3).*sqrt(-s) );         
        J1 = besselj( 1, theta(i,3).*sqrt(-s) );
        J2 = besselj( 2, theta(i,3).*sqrt(-s) );   
        J3 = besselj( 3, theta(i,3).*sqrt(-s) );
         
        psiNum = ( 3.*(theta(i,2)^3).*J3.*J2 + 2.*theta(i,2).*J1.*J2 +...
                  (theta(i,2)^2).*J0.*J3 );
                
        psiDen = ( (theta(i,2)^3).*J2.*J3 + theta(i,2).*J1.*J2 +...
                    3.*(theta(i,2)^2).*J0.*J3 + J0.*J1 );            
                
        psi = psiNum./psiDen;
        
        gamma(:,i) = sqrt( theta(i,4).*(s.^2) +...
                           (theta(i,1).*s.*sqrt(-s).*J0)./J1 +...
                           (theta(i,1).*theta(i,3).*(s.^2).*psi)./2 );
                                      
        Z0(:,i) = gamma(:,i)./( theta(i,5).*s );  


end

        
                         % END OF CODE %                          
