function [cablePar] = cable_parameters(cable, parameter)


%==========================================================================
%              GEOMETRIC AND ELECTROMAGNETIC CABLE PARAMETERS
%                                                                         
% Description:                                                            
%   This function provides the nominal values for the geometric and 
%   electromagnetic parameters related to a cable or a set of them.
%==========================================================================
% Input parameter(s):
%   cable          Real number of real-valued vector defining the gauge(s) 
%                  of the cable(s) of interest.  [mm]
%   parameter      Structure containing updated values for the geometric &
%                  electromagnetic parameters.
%==========================================================================
% Output parameter(s):
%  cablePar        Structure or array of structures with all info about
%                  a cable or a set of them.
%==========================================================================

%==========================================================================
% Author(s):
%   Roberto Menezes (RoMe / menegues@ufpa.br / UFPA)
%==========================================================================
% Change history (yyyy-mm-dd):
%   2009-07-15    (RoMe)        Created.
%==========================================================================
% Version: ??
%==========================================================================
% Known bugs (-) or limitations (*):                                      
%   None detected yet.
%==========================================================================
% Copyright (c) 2009 UFPA, Federal University of Para, Brazil.
%==========================================================================


%=== Definition and Pre-allocation variables:

nSec = length(cable);  % No. of line sections.
cablePar(nSec) = struct ('radius', [], 'distCenters', [], 'D2a', [], ... 
    'relPermittivity', [], 'permittivity0', [], 'relPermeability',  [], ...
    'permeability0', [], 'conductivity',[]);

    
for i = 1:nSec
   
    
%% ==== ELECTROMAGNETIC CABLE PARAMETERS ==================================

    if nargin == 2  % Parameters provided by the user. 

        CONDUC = parameter(i).sigma;  % Copper conductivy.   
        MI_REL = parameter(i).mi;  % Relative permeability.   
        EPSILON_REL = parameter(i).epsilon;  % Relative permissivity.     

    else  % Nominal values.

        CONDUC = 5.8e7;  % Copper conductivy (S/m).  
        MI_REL = 1;  % Polyethylene (PE) relative permeability.
        EPSILON_REL = 2.26;%2.595;  % 2.26; Polyethylene (PE) relative permissivity.        

    end

    MI_0 = (4*pi)*1e-7;  % Vacuum permeability (H/m).
    EPSILON_0 = 1e-9/(36*pi); %8.854e-12; % Free space permissivity (F/m). %1e-9/(36*pi);
    %MI = MI_REL * MI_0;  % Final permeability.
    %EP = EPSILON_REL * EPSILON_0;  % Final permittivity.

    % Obs:
    %
    %   Sources: a) ITU G996.1, pg. 28, table 10; 
    %            b) "Electromagnetic waves and antennas" --> Ch. 1/pg. XX.
    %----------------------------------------------------------------


%% ==== GEOMETRIC CABLE PARAMETERS ========================================


  %=== Nominal wire's radius (meters):
  %=== Source: ITU G996.1, pg. 28, table 10

    radiusITU  = [0.160e-3  0.200e-3  0.250e-3  0.315e-3  0.325e-3  0.450e-3];

    % Obs:
    %
    %   radiusITU(1)    0.32 mm cable's radius.
    %   radiusITU(2)    0.40 mm cable's radius (for 26 AWG as well).
    %   radiusITU(3)    0.50 mm cable's radius (for 24 AWG as well).
    %   radiusITU(4)    0.63 mm cable's radius.
    %   radiusITU(5)    0.65 mm cable's radius.
    %   radiusITU(6)    0.90 mm cable's radius.
    %----------------------------------------------------------------


  %=== Nominal insulation thickness (meters):
  %=== Source: ITU G996.1, pg. 28, table 10 

    insulITU = [0.050e-3  0.130e-3  0.150e-3  0.200e-3  0.25e-3  0.270e-3];

    % Obs:
    %
    %   insulITU(1)    0.32 mm insulator thickness.
    %   insulITU(2)    0.40 mm insulator thickness (for 26 AWG as well).
    %   insulITU(3)    0.50 mm insulator thickness (for 24 AWG as well).
    %   insulITU(4)    0.63 mm insulator thickness.
    %   insulITU(5)    0.65 mm insulator thickness.
    %   insulITU(6)    0.90 mm insulator thickness.
    %----------------------------------------------------------------


  %=== Nominal distance between the conductor centers (meters):

    distITU(1) = 2*( radiusITU(1) + insulITU(1) );
    distITU(2) = 2*( radiusITU(2) + insulITU(2) );
    distITU(3) = 2*( radiusITU(3) + insulITU(3) );
    distITU(4) = 2*( radiusITU(4) + insulITU(4) );
    distITU(5) = 2*( radiusITU(5) + insulITU(5) );
    distITU(6) = 2*( radiusITU(6) + insulITU(6) );

    % Obs: It is assumed a "twisted-pair" structure.
    %----------------------------------------------------------------


  %=== Nominal ratio D/2a (dimensionless):
   
    D2aITU = distITU./(2*radiusITU);
    
  
  %=== Update geometric parameters, if user provides values:
        
    [index] = find_cable_type(cable(i));  % Determine the cable type...
                                          %...of each line section.
                                          
    if nargin == 2  % Values provided by the user.        
        
        radiusITU(index) = parameter(i).radius;
        distITU(index) = parameter(i).distCenters;
        D2aITU(index) = parameter(i).D2a;
        
    end
        
    
%% ==== FULL DESCRIPTION OF THE CABLE(S) ==================================
  
    cablePar(i).radius = radiusITU(index);
    cablePar(i).distCenters = distITU(index);
    cablePar(i).D2a = D2aITU(index);
    cablePar(i).relPermittivity = EPSILON_REL;
    cablePar(i).permittivity0 = EPSILON_0;
    cablePar(i).relPermeability = MI_REL;
    cablePar(i).permeability0 = MI_0;
    cablePar(i).conductivity = CONDUC;
        
end


                        % END OF CODE %
                        
                        

%-----------------------------%
%----- Local Function(s) -----%
%-----------------------------%


function [index] = find_cable_type(cable)

        
    if cable == 0.32 || cable == 28  
            index = 1;        
        elseif cable == 0.40 || cable == 26
            index = 2;
        elseif cable == 0.50 || cable == 24
            index = 3;
        elseif cable == 0.63 || cable == 22
            index = 4;    
        elseif cable == 0.65
            index = 5;
        elseif cable == 0.90 || cable == 19
            index = 6;
        else
            error(['Wrong cable type!: ','section ']);
    end           
    
    