function [matT, varargout] = multiply_matrices(tridimMatrix, varargin)


%==========================================================================
%                      MULTIPLY MATRICES
%                                                                         
% Description:                                                            
%   This function calculates the product of a set of matrices (at least 
%   two). The code is able to deal with both a tridimensional matrix and a 
%   set of bidimensional matrices.
%==========================================================================
% Input parameter(s):
%   tridimMatrix            A tridimensional matrix or a bidimensional one.
%   varargin       In case "tridimMatrix" variable is a bidimensional matrix, 
%                  varargin will be the remaining set of bidimensional 
%                  matrices to be mutiplied.
%==========================================================================
% Output parameter(s):
%   matT1            Product of the provided matrices (ascending way).
%   matT2            Product of the provided matrices (descending way).
%==========================================================================

%==========================================================================
% Author(s):
%   Roberto Menezes (RoMe / menegues@ufpa.br / UFPA)
%==========================================================================
% Change history (yyyy-mm-dd):
%   2006-03-09    (RoMe)        Created.
%==========================================================================
% Version: ??
%==========================================================================
% Known bugs (-) or limitations (*):                                      
%   None detected yet.
%==========================================================================
% Copyright (c) 2006 UFPA, Federal University of Para, Brazil.
%==========================================================================


%% ==== CHECK INPUT =======================================================

%=== Check 1 - Only tridimensional matrices are acepted:

[dim1, dim2, dim3] = size(tridimMatrix);  % third dimension of the...
                                          %..."tridimMatrix" variable.
flag = ~isempty(varargin);


if dim3 < 2
    error('You must provide just a 3-dimension matrix!');
end

string1 = 'square';
string2 = 'freqType';
if dim1 == dim2
    matrixType = string1;
elseif dim2 == 4
    matrixType = string2;
else
    error('the matrices indexed by the 3rd dimension must be square or no. freq. x 4!' );
end


%% === MULTIPLY 3-DIMENSION MATRICES ======================================


if strcmpi(matrixType, string1)  % Square matrices.

    if flag && strcmpi(varargin,'forward')
        
        matT = tridimMatrix(:,:,1);  % Reference matrix for "forward"...
                                      % multiplication.
        for i = 1:(dim3 - 1)            
            matT = matT*tridimMatrix(:,:,i + 1);  % Forward ABCD matrix.            
        end        
        
    elseif flag && strcmpi(varargin,'backward')
        
        matT = tridimMatrix(:,:,dim3);  % Reference matrix for "backward"...
                                         %...multiplication.
        for i = 1:(dim3 - 1)            
            matT = matT*tridimMatrix(:,:,dim3 - i);  % Backward ABCD matrix.
        end        
        
    else
        
        matT = tridimMatrix(:,:,1); 
        matT2 = tridimMatrix(:,:,dim3); 
        
        for i = 1:(dim3 - 1)
            matT = matT*tridimMatrix(:,:,i + 1);  % Forward ABCD matrix.
            matT2 = matT2*tridimMatrix(:,:,dim3 - i);  % Backward ABCD matrix.
        end       
        varargout(1) = {matT2};
        
    end

    
elseif strcmpi(matrixType, string2)  % nfreq x 4 matrices.

    if flag && strcmpi(varargin,'forward')
        
        matT = tridimMatrix(:,:,1);  % Reference matrix for "forward"...
                                      % multiplication.
        for i = 1:(dim3 - 1)        
            matT = multiply(matT, tridimMatrix(:,:,i + 1));
        end        
        
    elseif flag && strcmpi(varargin,'backward')
        
        matT = tridimMatrix(:,:,dim3);  % Reference matrix for "backward"...
                                         %...multiplication.
        for i = 1:(dim3 - 1)        
            matT = multiply(matT, tridimMatrix(:,:,dim3 - i));
        end
        
    else
        
        matT = tridimMatrix(:,:,1); 
        matT2 = tridimMatrix(:,:,dim3); 
        
        for i = 1:(dim3 - 1)
            matT = multiply(matT, tridimMatrix(:,:,i + 1));
            matT2 = multiply(matT2, tridimMatrix(:,:,dim3 - i));
        end
        varargout = {matT2};
        
    end
    
    
end
    
    
  
                            % END OF CODE %
                            

                            
%-----------------------------%
%----- Local Function(s) -----%
%-----------------------------%


function Mtotal = multiply(matrix1, matrix2)
%%
   Mtotal(:,1) = matrix1(:,1).*matrix2(:,1) + matrix1(:,2).*matrix2(:,3);
   Mtotal(:,2) = matrix1(:,1).*matrix2(:,2) + matrix1(:,2).*matrix2(:,4);
   Mtotal(:,3) = matrix1(:,3).*matrix2(:,1) + matrix1(:,4).*matrix2(:,3);
   Mtotal(:,4) = matrix1(:,3).*matrix2(:,2) + matrix1(:,4).*matrix2(:,4);
   
   
