function Vl = pp_generate_TDR(Zin,frequencyAxis, pulseWidth, amplitude, Zs)

%================ TIME DOMAIN REFLECTOMETRY SIMULATION ===================%
%                                                                         %
% Author(s)     : Vinicius Lima (vinicius@ufpa.br)                        % 
% UFPA - Federal University of Para / Ericsson ASP Lab                    %
%                                                                         %  
% Abstract:                                                               %
%=========================================================================%
% Revision History:                                                       % 
%      2009-04-12  (ViLi)      Created.                                   % 
%%========================================================================%
% Input parameters:                                                       % 
%      Zin              Input Impedance (frequency domain).               %
%      frequencyAxis    Frequency axis (in Hertz).                        %
%      Zs               Impedance of the signal source.                   % 
%      pulseWidth       Width of the excitation pulse.                    %
%      amplitude        Amplitude of the excitation pulse.                %
%=========================================================================%
% Output parameters:                                                      %
%      Vl               TDR signal on frequency domain.                   %
%=========================================================================%
% Obs: The generator and the loop constitute a voltage divisor throught the 
% internal generator impedance and the input impedance of the loop. Then,
% the voltage across the input terminals of the loop, Vl(f), will be in
% terms of generator voltage, Vs(f), as follows: 
%
%           Vl(f) = Ir x Vs(f) , where Ir = (Zin)./(Zin + Zs).
%
% So, in the time domain version of Vl(f) will be expressed by the 
% convolution between the impedance ratio, Ir, and the time domain version
% of Vs(f). This is what will be done in the following...
%=========================================================================%

%% ==== SET DEAFAULT VALUES, IF NECESSARY

if nargin < 5
    Zs = 50; % Source impedance[Ohms]
end
if nargin < 4
    amplitude = 10; % TDR pulse amplitude (volts)
end
if nargin < 3
    pulseWidth = 1e-6; % TDR pulse width (seconds)
end


%% ==== SAMPLE FREQUENCY AND PERIOD

nSamples = length(Zin); 
nTimeSamples = 2*(nSamples - 1);         % N = number of time samples
bandwidth = frequencyAxis(nSamples - 1); % Bandwidth
samplePeriod = 1/(2*bandwidth);          % Sample period


%% ==== DEFINITION OF THE INPUT PULSE AND ITS ONE-SIDED SPECTRUM

vs = zeros(1,nTimeSamples);
nPulseSamples = floor(pulseWidth/samplePeriod);

% Rectangular pulse as input signal:
vs(nTimeSamples - nPulseSamples:nTimeSamples) = amplitude;
Vs = fft1(vs)';


%% ==== TDR: REFLECTOGRAM

Ir = Zin./(Zin + Zs);     
Vl = Ir.*Vs;                                % Frequency domain version
% [tdr, timeAxis] = ifft1(Vl, frequencyAxis); % Reflectogram