function [Z0, gamma] = KHM_model(secCable, f)

%==========================================================================
%                           KHM CABLE MODEL
%                                                                         
% Description:                                                            
%   This function provides the secondary parameters and gammaL of each
%   line section that characterize the line as a whole based on KHM cable model.
%==========================================================================
% Input parameter(s):
%   secCable       Row-vector containing the cable sequence for the line 
%                  under test, from CP to CO.                          
%   freq           Column-vector containing the frequency samples related  
%                  to the measured or simulated data.               [Hertz]
%==========================================================================
% Output parameter(s):
%   Z0             Characteristic impedance per line section. Matrix with 
%                  dimension (frequency samples x no. of sections).  [Ohms]
%   gamma          Propagation constant per line section. Matrix with
%                  dimension (frequency samples x no. of sections).  [km]



%% ===========================================================================
% Project       : G.fast/HFCC
% Author(s)     : Diogo Acatauassu (diogoaca@ufpa.br)
%
%% ===========================================================================
% Change History
%      2015-02-19 (DiAC) Created
%% ===========================================================================


%=== No. of samples and sections: 

nSamples = size(f, 1);  
nSec = size(secCable, 2);


%=== Pre-allocation for the primary and secundary parameters:

Z0 = zeros(nSamples, nSec);
gamma = zeros(nSamples, nSec);


for k = 1:nSec 


%% ===========================================================================
% Ericsson cables

% ELQXBE (1x4x0.5) - Extracted from a 50m cable
if(strcmp(secCable(k),'EAB_elqxbe_50m'));
    h1=125.3535;
    h2=5.6279e+03;
    k1=0.00196;
    k2=4.94725e-08;
    k3=3.03647e-05;
end

% ELQXBE (1x4x0.5) - Extracted from a 26m cable
if(strcmp(secCable(k),'EAB_elqxbe_26m'));
    h1=126.2974;
    h2=2.6528e+03;
    k1=0.00175;
    k2=5.47288e-08;
    k3=2.66477e-05;
end

%% ===========================================================================
% Unknown cables

% Typical CAT5 (4x2x0.5) - Extracted from a 62.9m cable
if(strcmp(secCable(k),'CAT5_62m'));
   h1=101.2393;
   h2=3.2608e+03;
   k1=0.00207;
   k2=2.39281e-08;
   k3=2.94153e-05;
end



%% ===========================================================================

gamma(:,k)=k1.*sqrt(f)+ k2.*f + 1i*k3.*f + 1i*k1.*sqrt(f) - 1i*k2*(2/pi).*f.*log(f);
Z0(:,k)=h1+(h2./(sqrt(f)))-(1i.*(h2./(sqrt(f))));

end