%This script creates the whole process of upconverting a signal
%and then downconverting with a carrier that has a frequency and
%phase offsets with respect to the transmitter carrier. This requires
%a relatively large oversampling factor (L).
%It uses PAM.
close all
fc=900e6; %nominal (assumed transmitter's) carrier frequency in Hz
freqOffset=50e3; %frequency offert
Rsym=20e6; %symbol rate in bauds
phaseOffset=1; %phase offset (to be imposed to the signal) in rad
%choose a sampling frequency high enough to acomodate 4*fc
L=ceil(6*fc/Rsym); % oversampling factor
Fs=L*Rsym; %sampling frequency in Hz
M=4; % modulation order ( cardinality of the set of symbols )
N=5000; % number of symbols to be generated
%assume some accuracy of oscillators
minFreqOffset=(fc/1e6)*25; %best accuracy is 25 ppm
maxFreqOffset=(fc/1e6)*200; %worst accuracy is 200 ppm
if freqOffset > maxFreqOffset
    error('freqOffset > maxFreqOffset')
end
if freqOffset < minFreqOffset
    error('freqOffset < minFreqOffset')
end
desiredResolutionHz=min(freqOffset/100,minFreqOffset/2); %desired FFT resolution in Hz
%if desiredResolutionHz<300
%    desiredResolutionHz=300;
%end
%% 1) Transmitter processing
%PAM generation:
indicesTx = floor(M*rand(1,N))+1; % random indices from [1, M]

%indicesTx = [1 4 2 3];

constellation = -(M-1):2:M-1; % symbols ..., -3, -1, 1, 3, 5, 7,...
m=constellation(indicesTx); % obtain N random symbols
m_upsampled = zeros(1,N*L); %pre-allocate space with zeros
m_upsampled(1:L:end)=m; % symbols with L -1 zeros in - between
rollOff=0.5; %roll-off factor for raised cosine filter
Dsym=3; % group delay, at rate Rsym (input symbols rate)
p=ak_rcosine(1,L,'fir/sqrt',rollOff,Dsym); %square-root raised cosine
gdelayRC=(length(p)-1)/2; %group delay of this even order RC filter
sbb=conv(m_upsampled,p); % convolve symbols with shaping pulse
n=0:length(sbb)-1; %discrete-time index
wc=(2*pi*fc)/Fs; %convert fc from Hz to radians
carrierTx=cos(wc*n); %transmitter carrier (assume phase is 0)
s=sbb.*carrierTx; %upconvert to carrier frequency
%% 2) Receiver processing
wOffset=(2*pi*freqOffset)/Fs; %convert freqOffset from Hz to radians

BWp=pi/L*(1+rollOff);
BWinHz=Fs*BWp/2*pi;

carrierRx=cos((wc+wOffset+BWp)*n+phaseOffset);%Rx carrier with offsets
r=2*s.*carrierRx; %downconvert to approximately baseband
%% 3) Estimate and correct offsets:
[estimatedFrequencyOffset,estimatedPhaseOffset,h,resolutionHz] = ...
    ak_pamCarrierRecoveryViaFFT(r,minFreqOffset,maxFreqOffset+BWinHz,Fs,desiredResolutionHz);

if 0
    estimatedPhaseOffset=0;
    estimatedFrequencyOffset=0;
end

%convert from Hz to radians:
wEstimatedFrequencyOffset = (2*pi*estimatedFrequencyOffset)/Fs;
%correct carrier offsets:
carOffSet=exp(-1j*(wEstimatedFrequencyOffset*n+estimatedPhaseOffset));
rc=r.*carOffSet; %generates complex signal with offsets subtracted
rc=real(rc); %convert to real signal account for factor of 2
%% 4) Evaluate results
%Compare with correct values:
phaseError = phaseOffset-estimatedPhaseOffset; %check error in phase
freqError = freqOffset-estimatedFrequencyOffset; %check error in freq.
%Demodulate and estimate symbol error rate (SER)
rf=conv(p,rc); %uses matched filter (note p is symmetric and real)
startSample=2*gdelayRC+1; %take in account the two filtering processes
symbolsRx = rf(startSample:L:startSample+L*(N-1)); %extract symbols
gainControl = sqrt(mean(abs(constellation).^2)/mean(abs(symbolsRx).^2));
symbolsRx = gainControl*symbolsRx; %normalize
indicesRx=1+ak_pamdemod(symbolsRx,M); %add one to make 1,...,M
disp(['SER = ' num2str(100*sum(indicesRx~=indicesTx)/N) '%']) %SER
disp(['Imposed phase offset = ' num2str(phaseOffset) ' rad'])
disp(['Estimated phase offset = ' num2str(estimatedPhaseOffset) ' rad'])
disp(['Phase error = ' num2str(phaseError) ' rad'])
disp(['Imposed frequency offset = ' num2str(freqOffset/1e3) ' kHz'])
disp(['Estimated frequency offset = ' num2str(estimatedFrequencyOffset/1e3) ' kHz'])
disp(['Frequency error = ' num2str(freqError) ' Hz'])
disp(['Desired FFT resolution=' num2str(desiredResolutionHz) ' Hz'])
disp(['FFT resolution = ' num2str(resolutionHz) ' Hz'])
disp(['Error in frequency is ' num2str(freqError*1e6/fc) ' ppm ' ...
    'of carrier = ' num2str(fc/1e3) ' kHz'])
disp(['Symbol rate = ' num2str(Rsym/1e6) ' Mbauds'])
disp('Normalized (for Matlab) values')
if 0
    disp(['Baud rate = ' num2str(Rsym/(Fs/2)) ' (normalized bauds)'])
    disp(['Freq. offset = ' num2str(freqOffset/(Fs/2)) ' (normalized Hz)'])
end
disp(['Product of freq. offset by Tsym = ' num2str(freqOffset/Rsym)])
disp(['This product above varies from 0.001 to 0.2 in papers'])
disp(['Searched freq. offset in range: [' num2str(minFreqOffset/1e3) ...
    ', ' num2str(maxFreqOffset/1e3) '] kHz']);
figure; plot(real(m),imag(m),'ro'); hold on
plot(real(symbolsRx ),imag(symbolsRx),'x')