function [ABCD, varargout] = ABCD_matrix(type, gammaL, Z0, varargin)


%==========================================================================
%                    ABCD MATRIX FROM SECONDARY PARAMETERS
%                                                                         
% Description:                                                            
%   This function calculates the forward and backward overall ABCD matrix 
%   for a line under test (LUT) based on its secondary parameters.
%==========================================================================
% Input parameter(s):
%   type           Cell array describing the type of each section of the 
%                  line under test (bridged-tap or serial).
%   gammaL         Propagation constant times length, per section. Matrix 
%                  with dimension (frequency samples x no. of sections).
%   Z0             Characteristic impedance per line section. Matrix 
%                  with dimension (frequency samples x no. of sections).
%==========================================================================
% Output parameter(s):
%   downABCD       Forward ABCD matrix of LUT. Matrix with dimension 
%                  (no. of frequency samples x 4).
%   upABCD         Backward ABCD matrix of LUT. Matrix with dimension 
%                  (no. of frequency samples x 4).
%==========================================================================

%==========================================================================
% Author(s):
%   Roberto Menezes (RoMe / menegues@ufpa.br / UFPA)
%==========================================================================
% Change history (yyyy-mm-dd):
%   2005-08-14    (RoMe)        Created.   
%   2009-05-21    (RoMe)        Cleaning up and rename of variables based 
%                               on predefined coding conventions.
%==========================================================================
% Version: ??
%==========================================================================
% Known bugs (-) or limitations (*):                                      
%   * None detected yet.
%==========================================================================
% Copyright (c) 2005 UFPA, Federal University of Para, Brazil.
%==========================================================================


%% ==== ABCD MATRIX OF EACH LINE SECTION ==================================

[nTones, nSec] = size(Z0);
ABCDsec = zeros(nTones, 4, nSec); % Pre-allocation of the ABCD per section.

for i = 1:nSec  
        
    if strcmp(type(i),'serial')
        ABCDsec(:,1,i) = cosh(gammaL(:,i)); 
        ABCDsec(:,2,i) = Z0(:,i).*sinh(gammaL(:,i));
        ABCDsec(:,3,i) = sinh(gammaL(:,i))./Z0(:,i);
        ABCDsec(:,4,i) = cosh(gammaL(:,i));
        
    elseif strcmp(type(i),'bridge-tap')
        ABCDsec(:,3,i) = 1./( Z0(:,i).*coth(gammaL(:,i)) );
        ABCDsec(:,1,i) = 1;
        ABCDsec(:,2,i) = 0;
        ABCDsec(:,4,i) = 1;
        
    else 
        error('unexpected value for "type" variable!');
    
    end 

end


%% ==== OVERALL ABCD MATRICES FOR DOWNSTREAM AND UPSTREAM =================

if nSec == 1
    ABCD = ABCDsec;
    varargout = {ABCDsec};
    
else
    [ABCD, temp] = multiplyABCDMatrices(ABCDsec, varargin);
    varargout = {temp};    
    
end
    


                           % END OF CODE %                                 
