function [Z0, gamma] = MAR2_model(secGauge, freq)


%==========================================================================
%                           MAR#2 CABLE MODEL
%                                                                         
% Description:                                                            
%   This function provides the secondary parameters and gammaL of each   
%   line section that characterize the line as a whole based on Marconi 
%   (MAR) 2 model.
%==========================================================================
% Input parameter(s):
%   secGauge       Row-vector containing the gauge sequence for the line 
%                  under test, from CP to CO.                          [mm]
%   freq           Column-vector containing the frequency samples related  
%                  to the measured or simulated data.               [Hertz]
%==========================================================================
% Output parameter(s):
%   Z0             Characteristic impedance per line section. Matrix with 
%                  dimension (frequency samples x no. of sections).  [Ohms]
%   gamma          Propagation constant per line section. Matrix with
%                  dimension (frequency samples x no. of sections).  [1/km]
%==========================================================================

%==========================================================================
% Author(s):
%   Roberto Menezes (RoMe / menegues@ufpa.br / UFPA)
%==========================================================================
% Change history (yyyy-mm-dd):
%   2007-18-04    (RoMe)        Created.
%==========================================================================
% Version: ??
%==========================================================================
% Known bugs (-) or limitations (*):                                      
%   None detected yet.
%==========================================================================
% Copyright (c) 2007 UFPA, Federal University of Para, Brazil.
%==========================================================================


%% ==== DEFINITION AND PRE-ALLOCATION OF VARIABLES ========================


%=== Angular frequency:

w = 2*pi*freq;                 


%=== No. of samples and sections: 

nSamples = size(freq, 1);  
nSec = size(secGauge, 2);
                                   

%=== Pre-allocation for the primary and secundary parameters:

Zs = zeros(nSamples, nSec);
Yp = zeros(nSamples, nSec);

Z0 = zeros(nSamples, nSec);
gamma = zeros(nSamples, nSec);


%% ==== MAR2 PARAMETERS PER GAUGE =========================================
 
%--- DC resistance: Ohm/km 
R0 = [400.666842, 277.623806, 179.460137, 111.100648, 54.753441];

%--- High frequency inductance: Henry/km 
Linf = 1e-6.*[411.961536, 421.576013, 516.017173, 471.310664, 502.965928];

%--- Capacitance at 1MHz: Fadaray/km 
C1M = 1e-9.*[39.639546, 49.792437, 49.840489, 44.685737, 39.855129];

%--- Proximity factor 
a = [2.156655, 1.470148, 1.705113, 1.948990, 2.446506];

%--- Shunt capacitance loss angle 
delta = [0.002840, 0.001123, 0.002647, 0.003210, 0.002673];

%--- Skin effect shape coefficients 
b = 2;
c = 2.765;


% Obs: Values extracted from "T. Nordstrom, S. Trautmann; ETSI STC TM; 
%       Proposal for Parametric ADSL Cable Models, February, pag. 4, 2002."
%
%   column 1 -->  0.32 mm.
%   column 2 -->  0.40 mm.
%   column 3 -->  0.50 mm.
%   column 4 -->  0.63 mm or 0.65 mm.
%   column 5 -->  0.90 mm.
%----------------------------------------------------------------


%% ==== CALCULATION OF ABCD MATRIX FOR EACH SECTION =======================
 
for k = 1:nSec 
    
    
  %=== Series impedance and shunt admitance:   
  
    if secGauge(k) == 0.32 || secGauge(k) == 28
        s = (1i.*freq)./(447.6.*R0(1));
        Zs(:,k) = 1i*w.*Linf(1) + R0(1).*( 0.25 + 0.75*sqrt(1 + a(1).*s.*(s + b)./(s + c)) );
        Yp(:,k) = ( 1i*w.*C1M(1) ).^(1 -2*delta(1)/pi);

    elseif secGauge(k) == 0.40 || secGauge(k) == 26
        s = (1i.*freq)./(447.6.*R0(2));
        Zs(:,k) = 1i*w.*Linf(2) + R0(2).*( 0.25 + 0.75*sqrt(1 + a(2).*s.*(s + b)./(s + c)) );
        Yp(:,k) = ( 1i*w.*C1M(2) ).^(1 -2*delta(2)/pi);

    elseif secGauge(k) == 0.50 || secGauge(k) == 24
        s = (1i.*freq)./(447.6.*R0(3));
        Zs(:,k) = 1i*w.*Linf(3) + R0(3).*( 0.25 + 0.75*sqrt(1 + a(3).*s.*(s + b)./(s + c)) );
        Yp(:,k) = ( 1i*w.*C1M(3) ).^(1 -2*delta(3)/pi);

    elseif secGauge(k) == 0.63 || secGauge(k) == 0.65 || secGauge(k) == 22
        s = (1i.*freq)./(447.6.*R0(4));
        Zs(:,k) = 1i*w.*Linf(4) + R0(4).*( 0.25 + 0.75*sqrt(1 + a(4).*s.*(s + b)./(s + c)) );
        Yp(:,k) = ( 1i*w.*C1M(4) ).^(1 -2*delta(4)/pi);

    elseif secGauge(k) == 0.90 || secGauge(k) == 19
        s = (1i.*freq)./(447.6.*R0(5));
        Zs(:,k) = 1i*w.*Linf(5) + R0(5).*( 0.25 + 0.75*sqrt(1 + a(5).*s.*(s + b)./(s + c)) );
        Yp(:,k) = ( 1i*w.*C1M(5) ).^(1 -2*delta(5)/pi);

    else
        error(['Wrong cable_type!: ','section ', num2str(k)]);
    end
        
  %=== Characteristic impedance and gamma*length: 
  
    Z0(:,k) = sqrt(Zs(:,k)./Yp(:,k));
    gamma(:,k) = sqrt( Zs(:,k).*Yp(:,k) );  % 1/km.
       
end 



                        % END OF CODE %                         
